//
// CSXBridge.jsx
//
//   For CS2 and CS3
//
//   This script allows CSX to be called from Bridge. The files that are
//   currently selected in Bridge are the ones that will be used by CSX.
//
//   CSX is called from the Tools > Photoshop > Contact Sheet X... menu item.
//   This location may change in a future release.
//
// Copy this file to the following directories as needed
//
//     On WinXP CS2
//           %CommonProgramFiles%\Adobe\StartupScripts
//
//     On WinXP CS3
//           %CommonProgramFiles%\Adobe\Startup Scripts CS3\Adobe Photoshop
//
//     On OSX CS2
//            /Library/Application Support/Adobe/StartupScripts
//
//     On OSX CS3
//            /Library/Application Support/Adobe/StartupScripts CS3/Adobe Photoshop
//
// Notes:
//   When Photoshop starts-up, this script gets executed and creates a
//     "csxbridge.ini" file in the xtools preferences folder. This file
//     contains the initialization information needed to run CSX from Bridge.
//
//   When CSX is invoked from Bridge, the current selection of files are
//     gathered and sent to Photoshop and used as the file set for CSX.
//     Bridge can then be used for other tasks while CSX is processing
//     the files.
//
// $Id: CSXBridge.jsx,v 1.33 2008/12/18 18:19:03 xbytor Exp $
// Copyright: (c)2008, xbytor
// License: http://creativecommons.org/licenses/LGPL/2.1
// Contact: xbytor@gmail.com
//
if (BridgeTalk.appName == "photoshop" || BridgeTalk.appName == "bridge") {
try {
CSXBridge = {};

//XXX Change these manually
CSXBridge.RELEASE = "1.4.2";
CSXBridge.UUID = "0be2b720-cd2e-11dd-ad8b-0800200c9a66";

CSXBridge.REV = CSXBridge.RELEASE.replace(/\./g, '_');
CSXBridge.SCRIPT = "CSX-" + CSXBridge.REV + ".jsx";
CSXBridge.MENU_TITLE = "Contact Sheet X v" + CSXBridge.RELEASE;

CSXBridge.SCRIPTS_FOLDER =
  new Folder(app.path + '/' +
             localize("$$$/ScriptingSupport/InstalledScripts=Presets/Scripts"));

CSXBridgeOptions = function(obj) {
  var self = this;

  self.script      = CSXBridge.SCRIPTS_FOLDER + '/' + CSXBridge.SCRIPT;
  self.dev         = false;
  self.timeout     = 24 *60 *60 *1000;   // 24 hour timeout
  self.logEnabled  = true;
  self.logFile     = Folder.userData + "/xtools/csxbridge.log"
  self.rev         = '';

  if (obj) {
    for (var idx in obj) {
      self[idx] = obj[idx];
    }
  }
};

CSXBridge.opts = undefined;

// for CSX in Photoshop
CSXBridge.callFromBridge = false; // a flag in case CSX needs to know
CSXBridge.files = undefined;

CSXBridge.isPhotoshop = function() {
  return BridgeTalk.appName == "photoshop";
};
CSXBridge.isPSCS2 = function() {
  return CSXBridge.isPhotoshop() && app.version.match(/^9\./);
};
CSXBridge.isPSCS3 = function() {
  return CSXBridge.isPhotoshop() && app.version.match(/^10\./);
};
CSXBridge.isPSCS4 = function() {
  return CSXBridge.isPhotoshop() && app.version.match(/^11\./);
};
CSXBridge.isBridge = function() {
  return BridgeTalk.appName == "bridge";
};
CSXBridge.isBridgeCS2 = function() {
  return CSXBridge.isBridge() && app.version.match(/^1\./);
};
CSXBridge.isBridgeCS3 = function() {
  return CSXBridge.isBridge() && app.version.match(/^2\./);
};
CSXBridge.isBridgeCS4 = function() {
  return CSXBridge.isBridge() && app.version.match(/^3\./);
};
CSXBridge.isCS2 = function() {
  return CSXBridge.isPSCS2() || CSXBridge.isBridgeCS2();
};
CSXBridge.isCS3 = function() {
  return CSXBridge.isPSCS3() || CSXBridge.isBridgeCS3();
};
CSXBridge.isCS4 = function() {
  return CSXBridge.isPSCS4() || CSXBridge.isBridgeCS4();
};

function toBoolean(s) {
  if (s == undefined) return false;
  if (s.constructor == Boolean) return s.valueOf();
  if (s.constructor == String)  return s.toLowerCase() == "true";
  return Boolean(s);
};

CSXBridge.init = function() {
  if (CSXBridge.opts) {
    return;
  }

  // read in the options
  var opts = CSXBridge.readIniFile();

  // setup the logging
  if (opts.logEnabled) {
    CSXBridge.log.enabled = toBoolean(opts.logEnabled);
  }
  if (opts.logFile) {
    opts.logFile = decodeURI(File(opts.logFile).fsName);
    CSXBridge.log.filename = opts.logFile;
  }

  opts.dev = toBoolean(opts.dev);

  // tada!

  if (CSXBridge.isCS2() || opts.dev) {
    CSXBridge.log("Start init " + app.name);
  }

  CSXBridge.opts = opts;

  // refresh the opts back to disk in case they weren't there
  if (CSXBridge.isCS2()) {
    CSXBridge.writeIniFile(opts);
  }

  try {
    if (CSXBridge.isPhotoshop()) {
      // For Photoshop, initialization consists of
      //   checking to see if we are in Dev mode and hardwiring
      //   the path to ContactSheetX in the ini file.
      //
      // If we are running in PSCS2, we also have to save out the
      //   ini file so that we can find the CSX script
      //

      // for photoshop init...
      opts.script = undefined;

      // Dev mode
      if (toBoolean(opts.dev)) {
        CSXBridge.log("Initializing CSXBridge/PS for Dev.");

        // if we're in dev mode use a master copy
        var devScript = new File("/c/work/csx/ContactSheetX.jsx");

        if (!devScript.exists) {
          // this is the path on OS X
          devScript = new File("~/work/csx/ContactSheetX.jsx");
        }

        if (devScript.exists) {
          // set the script and save it out in the options
          opts.script = devScript;
          CSXBridge.writeIniFile(opts);
          CSXBridge.log("Using script: " + devScript);
        }
      }

      if (CSXBridge.isCS2()) {
        // make sure the app path is in CSXBridge.opts
        // this will always be a Presets/Scripts folder
        if (!opts.path) {
          var path = CSXBridge.SCRIPTS_FOLDER;

          if (!path.exists) {
            var m = "Cannot find Photoshop scripts folder";
            CSXBridge.log(m);
            alert(m);
            return;
          }

          // store the Presets/Scripts path to the ini file
          opts.path = decodeURI(path.absoluteURI);

          CSXBridge.writeIniFile(opts);
          CSXBridge.log("script path: " + opts.path);
        }

        if (!opts.script || opts.rev != CSXBridge.REV) {
          var script = opts.path + '/' + CSXBridge.SCRIPT;

          // store the dist script filename
          opts.script = decodeURI(decodeURI(File(script).fsName));
          opts.rev = CSXBridge.REV;
          CSXBridge.writeIniFile(opts);
        }
      }
    }

    if (CSXBridge.isBridge()) {
      // Time for Bridge initialization...

      // Add CSX to the Tools > Photoshop menu
      if (opts.dev) {
        CSXBridge.log("Adding CSX to Bridge menu.");
      }

      // CSX is added to the top-level Tools menu..

//       CSXBridge.xtoolsSubmenu = MenuElement.create("menu",
//                                                    "xtools",

      var menuItemInfo = new Object();
      menuItemInfo.text = CSXBridge.MENU_TITLE;
      menuItemInfo.name = 'Tools/ContactSheetX';
      menuItemInfo.onDisplay = function() { this.enabled = true; }
      menuItemInfo.onSelect = CSXBridge.runFromBridge;

      var menuItem = MenuElement.find("ContactSheetX");
      if (!menuItem) {
        menuItem = MenuElement.create ('command',
                                       menuItemInfo.text,
                                       '-after submenu/VersionCue',
                                       menuItemInfo.name);
      }

      menuItem.onDisplay = menuItemInfo.onDisplay;
      menuItem.onSelect = menuItemInfo.onSelect;
    }

  } catch (e) {
    CSXBridge.log(e + '@' + e.line);
    alert("CSXBridge.jsx: " + e);
  }
};

CSXBridge.run = function(files) {
  try {
    if (!CSXBridge.isPhotoshop()) {
      return;
    }

    // This is the Photoshop side of the BridgeTalk wire
    CSXBridge.log("\nCSXBridge.run()");
    CSXBridge.callFromBridge = true;

    var opts = CSXBridge.opts;

    // If we are in CS3 or CS4 and not in Dev mode, we just run the
    // CSX command via the UUID
    if (CSXBridge.isCS3() || CSXBridge.isCS4() && !opts.dev) {
      CSXBridge.log("photoshop.runActionCommand(\"" +
                    CSXBridge.UUID + "\",\n\t" + files.join(",\n\t") + ");");
      photoshop.runActionCommand(CSXBridge.UUID, files);
      return;
    }

    //
    // The remainder of this is only for CS2 and Dev support
    // We have to eval() the script, which requires a different strategy
    //
    if (!opts.script) {
      var msg = "Unable to find Bridge-compatible version of ContactSheetX";
      CSXBridge.log(msg);
      alert(msg);
      return;
    }

    // Get the files
    if (files) {
      try { CSXBridge.files = eval(files); } catch (e) {}
    }

    CSXBridge.log("running script " + opts.script + " on " +  CSXBridge.files);
    var script = new File(opts.script);

    if (!script.exists) {
      var msg = "Unable to find script: " + decodeURI(script.fsName);
      CSXBridge.log(msg);
      throw msg;
    }

    if (CSXBridge.isPSCS2()) {
      script.open("r");
      var str = script.read();
      script.close();
      eval(str);

    } else {
      // untested
      $.evalFile(script, opts.timeout);
    }

  } catch (e) {
    if (e.number != 8007) {
      CSXBridge.log(e + '@' + e.line);
      alert("CSXBridge.jsx: " + e);
    }

  } finally {
    // Cleanup for the next run

    CSXBridge.callFromBridge = false;
    CSXBridge.files = undefined;
  }
};

CSXBridge.runFromBridge = function() {
  try {
    if (!CSXBridge.isBridge()) {
      return;
    }

    // This is the Bridge side of the BridgeTalk wire

    if (!BridgeTalk.isRunning("photoshop")) {
      alert("Please launch Photoshop before running this command.");
      return;
    }

    CSXBridge.log("\nCSXBridge.runFromBridge()");

    var files = CSXBridge.getSelectedFiles();

    CSXBridge.log("Sending to PS: [\n\t" + files.join(",\n\t") + "]");

    // Bring Photoshop to the foreground.
    BridgeTalk.bringToFront("photoshop");

    var btMessage = new BridgeTalk;
    btMessage.target = "photoshop";
    btMessage.body = "CSXBridge.run(" + files.toSource() + ");";

    btMessage.send();

    //CSXBridge.log("Message sent to PS");

  } catch (e) {
    if (!e.number == 8007) {
      CSXBridge.log(e + '@' + e.line);
      alert("CSXBridge.jsx: " + e);
    }
  }
};

CSXBridge.getSelectedFiles = function() {
  if (!CSXBridge.isBridge()) {
    return;
  }
  
  var files = [];
  var thumbnails = app.document.selections;
  
  for (var i = 0; i < thumbnails.length; i++ ) {
    var th = thumbnails[i];
    if (th && !th.container) {   // Don't add containers to the list yet
      files.push(th.spec);
    }
  }
  
  return files;
};

CSXBridge.readIniFile = function() {
  var f = CSXBridge.getIniFile();

  var opts;

  if (f.exists) {
    f.open("r", "TEXT", "????");
    f.lineFeed = 'unix';
    var str = f.read();
    f.close();
    opts = CSXBridge.fromIniString(str);
  }

  return new CSXBridgeOptions(opts);
};
CSXBridge.writeIniFile = function(obj) {
  var f = CSXBridge.getIniFile();
  var str = CSXBridge.toIniString(obj);

  f.open("w", "TEXT", "????");
  f.lineFeed = 'unix';
  f.writeln(str);
  f.close();
};

CSXBridge.getIniFile = function() {
  var appData = Folder.userData;
  var folder = new Folder(appData + "/xtools");

  if (!folder.exists) {
    folder.create();
  }

  return new File(folder + '/csxbridge.ini');
};

CSXBridge.toIniString = function(obj) {
  var str = '';
  for (var idx in obj) {
    if (idx.charAt(0) == '_') {         // private stuff
      continue;
    }
    if (idx == 'typename') {
      continue;
    }
    var val = obj[idx];

    if (val == undefined) {
      val = '';
    }

    if (val.constructor == String ||
        val.constructor == Number ||
        val.constructor == Boolean ||
        typeof(val) == "object") {
      str += (idx + ": " + val.toString() + "\n");
    }
  }
  return str;
};

CSXBridge.fromIniString = function(str, obj) {
  if (!obj) {
    obj = {};
  }
  var lines = str.split(/[\r\n]+/);

  var rexp = new RegExp(/([^:]+):(.*)$/);

  for (var i = 0; i < lines.length; i++) {
    var line = lines[i].trim();
    if (!line || line.charAt(0) == '#') {
      continue;
    }
    var ar = rexp.exec(line);
    if (!ar) {
      alert("Bad line in config file: \"" + line + "\"");
      return;
    }
    obj[ar[1].trim()] = ar[2].trim();
  }
  return obj;
};


CSXBridge.log = function(str) {
//   alert("LOG:" + str);
  if (!CSXBridge.log.enabled) {
    return;
  }
  if (!CSXBridge.log.filename) {
    return;
  }
  if (CSXBridge.log.filename == '_') {
    //$.writeln(new Date() + ' - ' + str);
    return;
  }

  var file = new File(CSXBridge.log.filename);

  CSXBridge.log.file = file;

  if (!file.open("e", "TEXT", "????"))  {
    throw ("Unable to open log file " + file + ": " + file.error);
  }

  file.lineFeed = 'unix';
  file.seek(0, 2); // jump to the end of the file
  if (!file.writeln(new Date() + " - " + str)) {
    throw Error("Unable to write to log file " + file + ": " + file.error);
  }
  
  file.close();
};
CSXBridge.log.enabled = false;
CSXBridge.log.filename = '_';

String.prototype.trim = function() {
  return this.replace(/^[\s]+|[\s]+$/g, '');
};


CSXBridge.init();

} catch (e) {
  CSXBridge.log(e + '@' + e.line);
  alert("CSXBridge.jsx: " + e);
}

 if (CSXBridge.opts.dev) {
   CSXBridge.log(app.name + " loaded");
 }

 if (CSXBridge.isBridge() && !CSXBridge.opts.dev) {
   try { File(CSXBridge.log.filename).remove(); } catch (e) {}
 }

}

"CSXBridge.jsx";
// EOF
